@@ -2,6 +2,7 @@ from typing import List, Any
 import os
 from urllib.parse import urlparse
 
+
 class Service:
     def __init__(self, bpm):
         self.bpm = bpm
@@ -110,47 +111,55 @@ class Service:
             "name": name,
             "filter": the_filter})
 
+
 class ContextInput(object):
-    def  __getitem__(self, item):
+    def __getitem__(self, item):
         return self.context.getInputValue(item)
 
     def __setitem__(self, key, value):
-        self.context.setInputValue(key,value)
+        self.context.setInputValue(key, value)
+
 
 class ContextJsonValue(object):
-    def  __getitem__(self, item):
-        return self.context.getJsonValue(self.item,item)
+    def __getitem__(self, item):
+        return self.context.getJsonValue(self.item, item)
 
     def __setitem__(self, key, value):
-        self.context.setJsonValue(self.item,key,value)
+        self.context.setJsonValue(self.item, key, value)
+
 
 class ContextJson(object):
     def __getattr__(self, item):
-        json_value= ContextJsonValue()
-        json_value.context=self.context
-        json_value.item=item
+        json_value = ContextJsonValue()
+        json_value.context = self.context
+        json_value.item = item
         try:
-            self.context.getValue(item+"!=null")
+            self.context.getValue(item + "!=null")
         except ValueError:
-            raise NameError(item) from None
+            if item == '_responseHeaders':
+                self.context.fakeContext[item] = {}
+            else:
+                raise NameError(item) from None
         return json_value
 
+
 class Context(object):
     input: ContextInput
     json: ContextJson
 
     def __init__(self, bpm):
-        super(Context,self).__setattr__('bpm', bpm)
+        super(Context, self).__setattr__('bpm', bpm)
         super(Context, self).__setattr__('input', ContextInput())
         super(Context, self).__setattr__('json', ContextJson())
-        self.input.context=self
-        self.json.context=self
+        super(Context, self).__setattr__('fakeContext', {})
+        self.input.context = self
+        self.json.context = self
 
     def __setattr__(self, key, value):
         if key in dir(self):
-            super(Context,self).__setattr__(key, value)
+            super(Context, self).__setattr__(key, value)
         else:
-            self.setValue(key,value)
+            self.setValue(key, value)
 
     def __getattr__(self, item):
         try:
@@ -158,32 +167,74 @@ class Context(object):
                 super(Context, self).__getattribute__(item)
             else:
                 return self.getValue(item)
-        except ValueError:
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                try:
+                    return self.fakeContext[item]
+                except KeyError:
+                    if item in os.environ:
+                        self.fakeContext[item] = os.environ.get(item)
+                        return os.environ.get(item)
             raise NameError(item) from None
 
-    def  __getitem__(self, item):
+    def __getitem__(self, item):
         try:
             return self.getValue(item)
-        except ValueError:
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                try:
+                    return self.fakeContext[item]
+                except KeyError:
+                    if item in os.environ:
+                        self.fakeContext[item] = os.environ.get(item)
+                        return os.environ.get(item)
             raise KeyError(item) from None
 
     def __setitem__(self, key, value):
-        return self.setValue(key,value)
+        try:
+            self.setValue(key, value)
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                self.fakeContext[key] = value
+            else:
+                raise x
 
     def setValue(self, variable: str, value: Any, address=None):
-        self.bpm.exec(script="_____context_____.setAttribute('" + variable + "',_____variable_____,100)",
-                      context={"_____variable_____": value},
-                      address=address)
+        try:
+            self.bpm.exec(script="_____context_____.setAttribute('" + variable + "',_____variable_____,100)",
+                          context={"_____variable_____": value},
+                          address=address)
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                self.fakeContext[variable] = value
+            else:
+                raise x
 
     def setJsonValue(self, variable: str, key: str, value: Any, address=None):
-        self.bpm.exec(script=variable + ('.put("%s",_____variable_____)' % key),
-                      context={"_____variable_____": value},
-                      address=address)
+        try:
+            self.bpm.exec(script=variable + ('.put("%s",_____variable_____)' % key),
+                          context={"_____variable_____": value},
+                          address=address)
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                if variable not in self.fakeContext.keys():
+                    self.fakeContext[variable] = {}
+                self.fakeContext[variable][key] = value
+            else:
+                raise x
 
     def setDictValue(self, variable: str, key: str, value: Any, address=None):
-        self.bpm.exec(script=variable + ('["%s"]=_____variable_____' % key),
-                      context={"_____variable_____": value},
-                      address=address)
+        try:
+            self.bpm.exec(script=variable + ('["%s"]=_____variable_____' % key),
+                          context={"_____variable_____": value},
+                          address=address)
+        except ValueError as x:
+            if str(x) == 'No address supplied':
+                if variable not in self.fakeContext.keys():
+                    self.fakeContext[variable] = {}
+                self.fakeContext[variable][key] = value
+            else:
+                raise x
 
     def setInputValue(self, key: str, value: Any, address=None):
         self.setJsonValue("input", key, value, address)
@@ -230,6 +281,18 @@ class ResourceService:
         return self.bpm.call("EBHBPMResourceService.listPaths", body={"path": path})
 
 
+class ReportService:
+    def __init__(self, bpm):
+        self.bpm = bpm
+
+    def renderReport(self, options: dict, reportData: dict):
+        return self.bpm.call("EBHBPMReportService.renderReport", body={"options": options, "reportData": reportData})
+
+    def renderReports(self, options: dict, reports: list):
+        return self.bpm.call("EBHBPMReportService.renderReports",
+                             body={"options": options, "reports": {"reports": reports}})
+
+
 class DocumentService:
     def __init__(self, bpm):
         self.bpm = bpm
@@ -330,6 +393,14 @@ class RuntimeService:
         self.bpm.call("EBHBPMRuntime.updateBusinessKey", body={
             "processInstanceId": processInstanceId, "businessKey": businessKey})
 
+    def getBusinessKey(self, processInstanceId: str):
+        return self.bpm.call("EBHBPMRuntime.getProcessBusinessKey", body={
+            "processInstanceId": processInstanceId})
+
+    def getProcessInstance(self, processInstanceId: str):
+        return self.bpm.call("EBHBPMRuntime.getProcessInstance", body={
+            "processInstanceId": processInstanceId})
+
     def setProcessInstanceName(self, processInstanceId: str, name: str):
         self.bpm.call("EBHBPMRuntime.setProcessInstanceName",
                       body={"processInstanceId": processInstanceId, "name": name})
@@ -353,12 +424,16 @@ class RuntimeService:
         self.bpm.call("EBHBPMRuntime.removeVariable",
                       body={"executionId": executionId, "variableName": variableName})
 
-    def signalEventReceived(self, executionId: str, signalName: str, processVariables: dict):
+    def signalEventReceived(self, executionId: str, signalName: str, processVariables=None):
+        if processVariables is None:
+            processVariables = {}
         self.bpm.call("EBHBPMRuntime.signalEventReceived",
                       body={"executionId": executionId, "signalName": signalName,
                             "processVariables": processVariables})
 
-    def messageEventReceived(self, executionId: str, messageName: str, processVariables: dict):
+    def messageEventReceived(self, executionId: str, messageName: str, processVariables=None):
+        if processVariables is None:
+            processVariables = {}
         self.bpm.call("EBHBPMRuntime.messageEventReceived",
                       body={"executionId": executionId, "messageName": messageName,
                             "processVariables": processVariables})
@@ -368,6 +443,11 @@ class ExecutionService:
     def __init__(self, bpm):
         self.bpm = bpm
 
+    def getBusinessKey(self):
+        process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
+        return self.bpm.call("EBHBPMRuntime.getProcessBusinessKey", body={
+            "processInstanceId": process_instance_id})
+
     def updateBusinessKey(self, businessKey: str):
         process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
         self.bpm.call("EBHBPMRuntime.updateBusinessKey", body={
@@ -393,14 +473,22 @@ class ExecutionService:
                              body={"executionId": process_instance_id, "variableName": variableName})
 
     def setVariable(self, variableName: str, value: Any):
-        process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
-        return self.bpm.call("EBHBPMRuntime.setVariable",
-                             body={"executionId": process_instance_id, "variableName": variableName, "value": value})
+        try:
+            self.bpm.exec(script="execution.setVariable(_____variable_____,_____value_____)",
+                          context={"_____variable_____": variableName, "_____value_____":value})
+        except ValueError as x:
+            process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
+            return self.bpm.call("EBHBPMRuntime.setVariable",
+                                 body={"executionId": process_instance_id, "variableName": variableName, "value": value})
 
     def removeVariable(self, variableName: str):
-        process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
-        self.bpm.call("EBHBPMRuntime.removeVariable",
-                      body={"executionId": process_instance_id, "variableName": variableName})
+        try:
+            self.bpm.exec(script="execution.removeVariable(_____variable_____)",
+                          context={"_____variable_____": variableName})
+        except ValueError as x:
+            process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
+            self.bpm.call("EBHBPMRuntime.removeVariable",
+                          body={"executionId": process_instance_id, "variableName": variableName})
 
     def getProcessInstanceId(self):
         process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
