@@ -4,8 +4,6 @@ import redflagbpm
 
 bpm=redflagbpm.BPMService()
 
-int
-
 # Leo los parámetros de configuración del mail en la BPM
 env=bpm.service.env()
 
@@ -27,7 +25,11 @@ vtl=bpm.service.execute("GARCPD/TEMPLATE",{})
 print(vtl)
 
 # Ejemplos de context
-bpm.context.nombre="Liorén"
+# bpm.context.nombre="Liorén"
+# bpm.context['nombre']="Liorén"
+
+print(bpm.context.nombre)
+print(bpm.context['nombre'])
 
 variables = {'persona.tipoId': 'CUIT',
              'persona.id': '30-51841045-4',
